@@ -22,6 +22,7 @@
 #define GLOM_DATASTRUCTURE_FIELD_H
 
 #include <libgdamm.h>
+#include "translatable_item.h"
 #include "layout/fieldformatting.h"
 #include "../sharedptr.h"
 
@@ -50,13 +51,18 @@ public:
     return (element->get_name() == m_strName);
   }
 
+  bool operator() (const sharedptr<const T_Element>& element)
+  {
+    return (element->get_name() == m_strName);
+  }
+
 protected:
   Glib::ustring m_strName;
 };
 
 
 //Field info, such as Name, Title, definitions, and, sometimes, contents.
-class Field
+class Field : public TranslatableItem
 {
 public:
   enum glom_field_type
@@ -79,15 +85,17 @@ public:
   bool operator==(const Field& src) const;
   bool operator!=(const Field& src) const;
 
+  Field* clone() const;
+
   glom_field_type get_glom_type() const;
   void set_glom_type(glom_field_type fieldtype);
 
 
   /// This forwards to the Gnome::Gda::FieldAttributes::get_name, so that we can use it in the same predicate template.
-  Glib::ustring get_name() const;
+  virtual Glib::ustring get_name() const;
 
   /// This forwards to the Gnome::Gda::FieldAttributes::set_name, for convenience
-  void set_name(const Glib::ustring& value);
+  virtual void set_name(const Glib::ustring& value);
 
   /// This forwards to the Gnome::Gda::FieldAttributes::get_auto_increment.
   bool get_auto_increment() const;
@@ -125,16 +133,11 @@ public:
   Gnome::Gda::Value get_data() const;
   void set_data(const Gnome::Gda::Value& value);
 
-  Glib::ustring get_title() const;
-  void set_title(const Glib::ustring& strTitle);
-
-  Glib::ustring get_title_or_name() const; //Title, or Name if there is no title.
-
   //Lookup stuff:
   bool get_is_lookup() const;
 
-  Glib::ustring get_lookup_relationship() const;
-  void set_lookup_relationship(const Glib::ustring& strRelationship);
+  sharedptr<Relationship> get_lookup_relationship() const;
+  void set_lookup_relationship(const sharedptr<Relationship>& strRelationship);
 
   Glib::ustring get_lookup_field() const;
   void set_lookup_field(const Glib::ustring& strField);
@@ -148,7 +151,7 @@ public:
   /** Escape the value so that it can be used in a SQL command.
    */
   Glib::ustring sql(const Gnome::Gda::Value& value) const;
-  
+
   /** Escape the value so that it can be used in a SQL command for a find.
    */
   Glib::ustring sql_find(const Gnome::Gda::Value& value) const;
@@ -192,7 +195,7 @@ public:
 
   static glom_field_type get_glom_type_for_gda_type(Gnome::Gda::ValueType gda_type);
   static Gnome::Gda::ValueType get_gda_type_for_glom_type(Field::glom_field_type glom_type);
-  
+
   static bool get_conversion_possible(glom_field_type field_type_src, glom_field_type field_type_dest);
 
   FieldFormatting m_default_formatting;
@@ -208,10 +211,10 @@ protected:
   //The gda type to be used for the glom type:
   typedef std::map<glom_field_type, Gnome::Gda::ValueType> type_map_glom_type_to_gda_type;
   static type_map_glom_type_to_gda_type m_map_glom_type_to_gda_type;
-  
+
   typedef std::list<glom_field_type> type_list_conversion_targets;
   typedef std::map<glom_field_type, type_list_conversion_targets> type_map_conversions;
- 
+
   static type_map_type_names m_map_type_names; //These are canonical, for internal use.
   static type_map_type_names m_map_type_names_ui; //These are translated.
   static type_map_conversions m_map_conversions; //Map of types to list of possibnle conversion targets types.
@@ -221,8 +224,9 @@ protected:
   Gnome::Gda::FieldAttributes m_field_info;
 
   Gnome::Gda::Value m_data; //Not used much.
-  Glib::ustring m_strTitle;
-  Glib::ustring m_strLookupRelationship, m_strLookupField;
+
+  sharedptr<Relationship> m_lookup_relationship;
+  Glib::ustring m_strLookupField;
   Glib::ustring m_calculation;
   bool m_visible; //Whether it will be shown to the user.
 };
