@@ -26,22 +26,13 @@ Dialog_ChooseField::Dialog_ChooseField(BaseObjectType* cobject, const Glib::RefP
 : Gtk::Dialog(cobject),
   m_combo_relationship(0),
   m_button_select(0),
-  m_checkbutton_editable(0),
   m_treeview(0),
   m_document(0)
 {
   refGlade->get_widget("button_select", m_button_select);
-  refGlade->get_widget("checkbutton_editable", m_checkbutton_editable);
   refGlade->get_widget_derived("combobox_relationship", m_combo_relationship);
   m_combo_relationship->signal_changed().connect(sigc::mem_fun(*this, &Dialog_ChooseField::on_combo_relationship_changed));
 
-  //Numeric formatting:
-  refGlade->get_widget("frame_numeric_format", m_frame_numeric_format);
-  refGlade->get_widget("checkbutton_format_thousands", m_checkbox_format_use_thousands);
-  refGlade->get_widget("checkbutton_format_use_decimal_places", m_checkbox_format_use_decimal_places);
-  refGlade->get_widget("entry_format_decimal_places", m_entry_format_decimal_places);
-  refGlade->get_widget_derived("entry_currency_symbol", m_entry_currency_symbol);
-
   refGlade->get_widget("treeview_fields", m_treeview);
 
   if(m_treeview)
@@ -51,6 +42,7 @@ Dialog_ChooseField::Dialog_ChooseField(BaseObjectType* cobject, const Glib::RefP
 
     m_treeview->append_column( _("Name"), m_ColumnsFields.m_col_name );
     m_treeview->append_column( _("Title"), m_ColumnsFields.m_col_title );
+    m_model->set_sort_column(m_ColumnsFields.m_col_name, Gtk::SORT_ASCENDING);
 
     m_treeview->signal_row_activated().connect( sigc::mem_fun(*this, &Dialog_ChooseField::on_row_activated) );
 
@@ -68,49 +60,40 @@ Dialog_ChooseField::~Dialog_ChooseField()
 {
 }
 
-void Dialog_ChooseField::set_document(Document_Glom* document, const Glib::ustring& table_name,  const LayoutItem_Field& field)
+void Dialog_ChooseField::set_document(Document_Glom* document, const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field)
 {
   set_document(document, table_name);
 
   //Select the current relationship at the start:
 
-  if(field.get_has_relationship_name())
+  if(field && field->get_has_relationship_name())
   {
-    m_combo_relationship->set_active_text( field.get_relationship_name() );
+    m_combo_relationship->set_selected_relationship( field->get_relationship() );
   }
   else
-    m_combo_relationship->set_active_text(table_name);
+    m_combo_relationship->set_selected_parent_table(table_name); //TODO: Show the title too.
 
   //Select the current field at the start:
-  const Glib::ustring field_name = field.get_name();
-
-  //Get the iterator for the row:
-  Gtk::TreeModel::iterator iterFound = m_model->children().end();
-  for(Gtk::TreeModel::iterator iter = m_model->children().begin(); iter != m_model->children().end(); ++iter)
+  if(field)
   {
-    if(field_name == (*iter)[m_ColumnsFields.m_col_name])
+    const Glib::ustring field_name = field->get_name();
+
+    //Get the iterator for the row:
+    Gtk::TreeModel::iterator iterFound = m_model->children().end();
+    for(Gtk::TreeModel::iterator iter = m_model->children().begin(); iter != m_model->children().end(); ++iter)
     {
-      iterFound = iter;
-      break;
+      if(field_name == (*iter)[m_ColumnsFields.m_col_name])
+      {
+        iterFound = iter;
+        break;
+      }
+    }
+
+    if(iterFound != m_model->children().end())
+    {
+      m_treeview->get_selection()->select(iterFound);
     }
   }
-
-  if(iterFound != m_model->children().end())
-  {
-    m_treeview->get_selection()->select(iterFound);
-  }
-
-  m_checkbutton_editable->set_active( field.get_editable() );
-
-
-  m_checkbox_format_use_thousands->set_active( field.m_numeric_format.m_use_thousands_separator );
-  m_checkbox_format_use_decimal_places->set_active( field.m_numeric_format.m_decimal_places_restricted );
-
-  char pchText[10] = {0};
-  sprintf(pchText, "%d", field.m_numeric_format.m_decimal_places);
-  m_entry_format_decimal_places->set_text(Glib::ustring(pchText));
-
-  m_entry_currency_symbol->get_entry()->set_text(field.m_numeric_format.m_currency_symbol);
 }
 
 void Dialog_ChooseField::set_document(Document_Glom* document, const Glib::ustring& table_name)
@@ -118,8 +101,6 @@ void Dialog_ChooseField::set_document(Document_Glom* document, const Glib::ustri
   m_document = document;
   m_table_name = table_name;
 
-  m_checkbutton_editable->set_active( true ); //An appropriate default.
-
   if(!m_document)
   {
     g_warning("Dialog_ChooseField::set_document(): document is null");
@@ -134,23 +115,16 @@ void Dialog_ChooseField::set_document(Document_Glom* document, const Glib::ustri
   if(document)
   {
     //Fill the list of relationships:
-    m_combo_relationship->clear_text();
-    Document_Glom::type_vecRelationships vecRelationships = document->get_relationships(table_name);
+    const Document_Glom::type_vecRelationships vecRelationships = document->get_relationships(table_name);
 
     //Add a special option for the current table:
-    m_combo_relationship->append_text(table_name);
+    m_combo_relationship->set_display_parent_table(table_name, document->get_table_title(table_name));
 
     //Add the relationships for this table:
-    if(!vecRelationships.empty())
-      m_combo_relationship->append_separator();
-
-    for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); ++iter)
-    {
-      m_combo_relationship->append_text(iter->get_name());
-    }
+    m_combo_relationship->set_relationships(vecRelationships, table_name, document->get_table_title(table_name));
 
     //Set the table name and title:
-    m_combo_relationship->set_active_text(table_name);
+    m_combo_relationship->set_selected_parent_table(table_name);
 
     //Fill the treeview:
     m_model->clear();
@@ -159,9 +133,11 @@ void Dialog_ChooseField::set_document(Document_Glom* document, const Glib::ustri
     {
       Gtk::TreeModel::iterator iterRow = m_model->append();
       Gtk::TreeModel::Row row = *iterRow;
-      row[m_ColumnsFields.m_col_name] = iter->get_name();
-      row[m_ColumnsFields.m_col_title] = iter->get_title();
-      row[m_ColumnsFields.m_col_field] = *iter;
+
+      sharedptr<Field> field = *iter;
+      row[m_ColumnsFields.m_col_name] = field->get_name();
+      row[m_ColumnsFields.m_col_title] = field->get_title();
+      row[m_ColumnsFields.m_col_field] = field;
     }
   }
 }
@@ -187,26 +163,13 @@ void Dialog_ChooseField::select_item(const Field& field)
 }
 */
 
-bool Dialog_ChooseField::get_field_chosen(LayoutItem_Field& field) const
+sharedptr<LayoutItem_Field> Dialog_ChooseField::get_field_chosen() const
 {
   //Don't initialize output argument,
   //because it might contain other useful data.
   //TODO: Store a LayoutItem_Field as a member.
 
-
-  //Relationship:
-  Glib::ustring relationship_name = m_combo_relationship->get_active_text();
-  if(relationship_name == m_table_name)
-    relationship_name.clear();
-
-  //field.set_relationship_name(relationship_name);
-
-  if(!relationship_name.empty())
-  {
-    m_document->get_relationship(m_table_name, relationship_name, field.m_relationship);
-  }
-
-  field.set_editable( m_checkbutton_editable->get_active() );
+  sharedptr<LayoutItem_Field> field;
 
   //Field:
   Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview->get_selection();
@@ -215,24 +178,25 @@ bool Dialog_ChooseField::get_field_chosen(LayoutItem_Field& field) const
     Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
     if(iter)
     {
+      field = sharedptr<LayoutItem_Field>::create(); //TODO_Translation: Preserve existing.
+
+      //Relationship:
+      //Note that a null relationship means that the parent table was selected instead.
+      sharedptr<Relationship> relationship = m_combo_relationship->get_selected_relationship();
+
+      //field.set_relationship_name(relationship_name);
+
+      field->set_relationship(relationship);
+
+
       Gtk::TreeModel::Row row = *iter;
-      field.set_name(row[m_ColumnsFields.m_col_name]);
-      field.m_field = row[m_ColumnsFields.m_col_field];
-
-      //Numeric Formatting:
-      field.m_numeric_format.m_use_thousands_separator = m_checkbox_format_use_thousands->get_active();
-      field.m_numeric_format.m_decimal_places_restricted = m_checkbox_format_use_decimal_places->get_active();
-
-      const Glib::ustring strDecPlaces = m_entry_format_decimal_places->get_text();
-      field.m_numeric_format.m_decimal_places = atoi(strDecPlaces.c_str());
-
-      field.m_numeric_format.m_currency_symbol = m_entry_currency_symbol->get_entry()->get_text();
-
-      return true;
+      sharedptr<Field> field_details = row[m_ColumnsFields.m_col_field];
+      field->set_full_field_details(field_details);
+      field->set_name(row[m_ColumnsFields.m_col_name]);
     }
   }
 
-  return false;
+  return field;
 }
 
 void Dialog_ChooseField::on_row_activated(const Gtk::TreePath& /* path */, Gtk::TreeViewColumn* /* view_column */)
@@ -242,7 +206,7 @@ void Dialog_ChooseField::on_row_activated(const Gtk::TreePath& /* path */, Gtk::
 
 void Dialog_ChooseField::on_combo_relationship_changed()
 {
-  Glib::ustring relationship_name = m_combo_relationship->get_active_text();
+  sharedptr<Relationship> relationship = m_combo_relationship->get_selected_relationship();
 
   Document_Glom* pDocument = m_document;
   if(pDocument)
@@ -250,14 +214,11 @@ void Dialog_ChooseField::on_combo_relationship_changed()
     //Show the list of fields from this relationship:
 
     Document_Glom::type_vecFields vecFields;
-    if(relationship_name == m_table_name)
+    if(!relationship)
       vecFields = pDocument->get_table_fields(m_table_name);
     else
     {
-      Relationship relationship;
-      pDocument->get_relationship(m_table_name, relationship_name, relationship);
-
-      vecFields = pDocument->get_table_fields(relationship.get_to_table());
+      vecFields = pDocument->get_table_fields(relationship->get_to_table());
     }
 
     m_model->clear();
@@ -265,9 +226,11 @@ void Dialog_ChooseField::on_combo_relationship_changed()
     {
       Gtk::TreeModel::iterator iterRow = m_model->append();
       Gtk::TreeModel::Row row = *iterRow;
-      row[m_ColumnsFields.m_col_name] = iter->get_name();
-      row[m_ColumnsFields.m_col_title] = iter->get_title();
-      row[m_ColumnsFields.m_col_field] = *iter;
+
+      sharedptr<Field> field = *iter;
+      row[m_ColumnsFields.m_col_name] = field->get_name();
+      row[m_ColumnsFields.m_col_title] = field->get_title();
+      row[m_ColumnsFields.m_col_field] = field;
     }
 
   }
@@ -281,6 +244,7 @@ void Dialog_ChooseField::on_treeview_selection_changed()
     Gtk::TreeModel::iterator iter = refSelection->get_selected();
     if(iter)
     {
+      /*
       Gtk::TreeModel::Row row = *iter;
       const Field& field = row[m_ColumnsFields.m_col_field];
       const bool is_numeric = (field.get_glom_type() == Field::TYPE_NUMERIC);
@@ -288,6 +252,7 @@ void Dialog_ChooseField::on_treeview_selection_changed()
         m_frame_numeric_format->show();
       else
         m_frame_numeric_format->hide();
+      */
     }
   }
 }
